//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * シンプルフレームワークヘッダ
 * @author Junpee
 */

#ifndef SIMPLE_FRAMEWORK_H_
#define SIMPLE_FRAMEWORK_H_

#include <Framework/Utility/WindowCreator.h>

// 以下のファイルは頻繁に使用するのであらかじめインクルードしておく
#include <Input/Keyboard/Keyboard.h>
#include <Input/Mouse/Mouse.h>
#include <Core/Utility/Random.h>

namespace Lamp{

class FPSController;

//------------------------------------------------------------------------------
/**
 * シンプルフレームワーク
 *
 * 最低限必要な処理を詰め込んだ小さなフレームワーク。基本的にフレームワークはLampに
 * とっつきやすくするため、用意されています（例えるならコード量の少ないd3dapp）。
 * ちゃんとしたアプリケーションを組む場合は、このフレームワークを参考にしてプログラム
 * を1から書きはじめ、だんだん大きくしていけばよいでしょう。
 * その場合は空っぽのオーバライドメソッドをフレームワークメソッドと置き換えるように
 * するとプログラムがシンプルになります。
 */
class SimpleFramework{
public:
	//--------------------------------------------------------------------------
	// 生成、破棄
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 * @param name アプリケーション名
	 */
	SimpleFramework(const String& name);

	/**
	 * デストラクタ
	 */
	virtual ~SimpleFramework();

	/**
	 * 実行
	 * @param instance インスタンスハンドル
	 * @return 成功すれば0を返す
	 */
	virtual int execute(HINSTANCE instance);

protected:
	//--------------------------------------------------------------------------
	// オーバーライドメソッド
	//--------------------------------------------------------------------------
	/**
	 * 初期化
	 * @return 初期化に成功すればtrueを返す
	 */
	virtual bool initialize(){ return true; }

	/**
	 * 後始末
	 */
	virtual void finalize(){}

	/**
	 * 実行
	 */
	virtual void run(){}

	/**
	 * レンダリング準備
	 */
	virtual void renderSetup(){}

	/**
	 * レンダリング
	 */
	virtual void render(){}

	//--------------------------------------------------------------------------
	// フレームワークメソッド
	//--------------------------------------------------------------------------
	/**
	 * フレームワークの初期化
	 * @param instanceHandle インスタンスハンドル
	 * @return 初期化に成功すればtrueを返す
	 */
	virtual bool frameworkInitialize(HINSTANCE instanceHandle);

	/**
	 * フレームワークの実行
	 * @return 実行結果。WinMainの返り値
	 */
	virtual int frameworkExecute();

	/**
	 * フレームワークの後始末
	 */
	virtual void frameworkFinalize();

	//--------------------------------------------------------------------------
	// ループ内の処理
	//--------------------------------------------------------------------------
	/**
	 * メインループ
	 */
	virtual void mainLoop();

	/**
	 * フレームワーク実行
	 */
	virtual void frameworkRun();

	/**
	 * フレームワークレンダリング準備
	 */
	virtual void frameworkRenderSetup();

	/**
	 * フレームワークプレゼンテーション
	 * @return デバイスがロストしていればtrue
	 */
	virtual bool frameworkPresentation();

	/**
	 * フレームワークレンダリング
	 */
	virtual void frameworkRender();

	//--------------------------------------------------------------------------
	// ウィンドウプロシージャ関連
	//--------------------------------------------------------------------------
	/**
	 * ウィンドウプロシージャ
	 * @param windowHandle ウィンドウハンドル
	 * @param message メッセージ
	 * @param wParam wメッセージパラメータ
	 * @param lParam lメッセージパラメータ
	 * @return メッセージを処理し、それ以上の処理が必要無いならば0以外を返す。
	 */
	virtual LRESULT windowProcedure(
		HWND windowHandle, u_int message, WPARAM wParam, LPARAM lParam){
		return 0;
	}

	/**
	 * フレームワークウィンドウプロシージャ
	 * @param windowHandle ウィンドウハンドル
	 * @param message メッセージ
	 * @param wParam wメッセージパラメータ
	 * @param lParam lメッセージパラメータ
	 * @return メッセージを処理し、それ以上の処理が必要無いならば0以外を返す。
	 */
	virtual LRESULT frameworkWindowProcedure(
		HWND windowHandle, u_int message, WPARAM wParam, LPARAM lParam);

public:
	//--------------------------------------------------------------------------
	/**
	 * 登録用ウィンドウプロシージャ
	 * @param windowHandle ウィンドウハンドル
	 * @param message メッセージ
	 * @param wParam wメッセージパラメータ
	 * @param lParam lメッセージパラメータ
	 */
	static LRESULT CALLBACK registrationWindowProcedure(
		HWND windowHandle, u_int message, WPARAM wParam, LPARAM lParam);

protected:
	//--------------------------------------------------------------------------
	/// 名前
	String name_;
	/// ウィンドウハンドル
	HWND windowHandle_;
	/// ウィンドウ作成パラメータ
	WindowCreator::CreateParameter createWindowParameter_;
	/// ウィンドウ最小サイズ
	DimensionI minimumWindowSize_;
	/// 背景色
	Color4c backGroundColor_;

	/// FPSコントローラ
	FPSController* fpsController_;
	/// キーボード
	Keyboard* keyboard_;
	/// マウス
	Mouse* mouse_;
	/// フルスクリーンで開始するか
	bool startFullscreen_;

	/// インスタンス
	static SimpleFramework* instance_;

private:
	// コピーコンストラクタの隠蔽
	SimpleFramework(const SimpleFramework& copy);

	// 代入コピーの隠蔽
	void operator =(const SimpleFramework& copy);

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of SIMPLE_FRAMEWORK_H_
//------------------------------------------------------------------------------

